/*!
	@file	MyCFGACIData.cpp
	@author	Copyright (C) 1998-2008 Eastman Kodak Company.
	@author	All rights reserved.
	@date	01-Oct-2008
*/

#include "StdAfx.h"
#include "MyCFGACIData.h"

//	CMyCFGACIData
//******************************************************************************
CMyCFGACIData::CMyCFGACIData()
{
}

CMyCFGACIData::~CMyCFGACIData()
{
}

//	derived from CDFAPIDataHelper
//------------------------------------------------------------------------------
void CMyCFGACIData::Reset()
{
	m_lContryIndex		= 0;
	m_lAllowUserTyping	= 0;
	m_lControlType		= 0;
}

// The configuration data are load/save as a block of memory in our system.
// db (type: DFAPI_DATA_TYPE) represent this block of memory. 
// ptr (type: DWORD) indicates the current write/read position in this block of memory.

// OnToDataBlock and OnFromDataBlock are 2 functions that do the conversion work between
// a block of memory and the program data variables.

// Notes: You should write the variables to the memory block in the same sequence you expect to read them
// The c_signature value is for validation only.

// Other similiar functions as W_DWORD64(), W_long(), W_string(), R_DWORD64(), R_long(), R_string()...
// can be found in DFAPIDataHelper.h. 

BOOL CMyCFGACIData::OnToDataBlock(DWORD& ptr, DFAPI_DATA_TYPE& db)
{
	// Put a random DWORD64 number as signature in the block data of this kind of plugin.
	// Every plugin signature shoud be different!
	const DWORD64 c_signature = 0x13413234546109;
	W_DWORD64(ptr, db, c_signature);

	// Write(save) data members
	W_long(ptr, db, m_lContryIndex);
	W_long(ptr, db, m_lAllowUserTyping);
	W_long(ptr, db, m_lControlType);

	return TRUE;
}

BOOL CMyCFGACIData::IsValidData()
{
	// Validate the data values

	// In this example we need to ensure that the m_lControlType is 0, 1 or 2

	BOOL bOK = FALSE;

	switch( m_lControlType )
	{		// ensure its a valid control ID
	case 0:		
	case 1:
	case 2:	bOK = TRUE; break;
	default: bOK = FALSE; break;
	}

	return bOK;
}

BOOL CMyCFGACIData::OnFromDataBlock(DWORD& ptr, DFAPI_DATA_TYPE& db)
{
	//c_signature is the same as above. 
	const DWORD64 c_signature = 0x13413234546109;
	DWORD64 signature;

	//Before read actual data, check the signature
	R_DWORD64(ptr, db, signature);
	if (signature != c_signature) 
		return FALSE;

	// data members hould be read in the same order they were written
	//Read actual data members
	R_long(ptr, db, m_lContryIndex);
	R_long(ptr, db, m_lAllowUserTyping);
	R_long(ptr, db, m_lControlType);

	return TRUE;
}