/*!
	@file	MyPlugin.cpp
	@author	Copyright (C) 1998-2008 Eastman Kodak Company.
	@author	All rights reserved.
	@date	01-Oct-2008
*/

#include "StdAfx.h"
#include "MyPlugin.h"
#include "MyCFGACIData.h"
#include "MyCFGACIUI.h"

#include "SelectStateDlg.h"


// The data could be from a database, Here in example we just use a predefined list for simplicity.
// Here just an example to show how you can list predefine value for user to choose when user doing index.
// And how the plugin can validate the user's input is correct or not.

#define		COUNTRY_US		0
#define		COUNTRY_CN		1

static struct State_Country{
	int			nCountry;
	TCHAR*		pchState;
}g_map[] = {
	{COUNTRY_US, _T("Alabama")},
	{COUNTRY_US, _T("Alaska")},
	{COUNTRY_US, _T("Arizona")},
	{COUNTRY_US, _T("Arkansas")},
	{COUNTRY_US, _T("California")},
	{COUNTRY_US, _T("Colorado")},
	{COUNTRY_US, _T("Connecticut")},
	{COUNTRY_US, _T("Delaware")},
	{COUNTRY_US, _T("Florida")},
	{COUNTRY_US, _T("Georgia")},
	{COUNTRY_US, _T("Hawii")},
	{COUNTRY_US, _T("Idaho")},
	{COUNTRY_US, _T("Illinois")},
	{COUNTRY_US, _T("Indiana")},
	{COUNTRY_US, _T("Lowa")},
	{COUNTRY_US, _T("Kansas")},
	{COUNTRY_US, _T("Kentucky")},
	{COUNTRY_US, _T("Louisiana")},
	{COUNTRY_US, _T("Maine")},
	{COUNTRY_US, _T("Maryland")},
	{COUNTRY_US, _T("Massachusetts")},
	{COUNTRY_US, _T("Michigan")},
	{COUNTRY_US, _T("Minnesota")},
	{COUNTRY_US, _T("Mississippi")},
	{COUNTRY_US, _T("Missouri")},
	{COUNTRY_US, _T("Montana")},
	{COUNTRY_US, _T("Nebraska")},
	{COUNTRY_US, _T("Nevada")},
	{COUNTRY_US, _T("New Hampshire")},
	{COUNTRY_US, _T("New Jersey")},
	{COUNTRY_US, _T("New Mexico")},
	{COUNTRY_US, _T("New York")},
	{COUNTRY_US, _T("North Carolina")},
	{COUNTRY_US, _T("North Dakota")},
	{COUNTRY_US, _T("Ohio")},
	{COUNTRY_US, _T("Oklahoma")},
	{COUNTRY_US, _T("Oregon")},
	{COUNTRY_US, _T("PennsyIvania")},
	{COUNTRY_US, _T("Rhode Island")},
	{COUNTRY_US, _T("South Carolina")},
	{COUNTRY_US, _T("South Dakota")},
	{COUNTRY_US, _T("Tennessee")},
	{COUNTRY_US, _T("Texas")},
	{COUNTRY_US, _T("Utah")},
	{COUNTRY_US, _T("Vermont")},
	{COUNTRY_US, _T("Virginia")},
	{COUNTRY_US, _T("Washington")},
	{COUNTRY_US, _T("West Virginia")},
	{COUNTRY_US, _T("Wisconsin")},
	{COUNTRY_US, _T("Wyoming")},

	{COUNTRY_CN, _T("Anhui")},
	{COUNTRY_CN, _T("Fujian")},
	{COUNTRY_CN, _T("Gansu")},
	{COUNTRY_CN, _T("Guangdong")},
	{COUNTRY_CN, _T("Guizhou")},
	{COUNTRY_CN, _T("Hainan")},
	{COUNTRY_CN, _T("Hebei")},
	{COUNTRY_CN, _T("Heilongjiang")},
	{COUNTRY_CN, _T("Henan")},
	{COUNTRY_CN, _T("Hubei")},
	{COUNTRY_CN, _T("Hunan")},
	{COUNTRY_CN, _T("Jiangsu")},
	{COUNTRY_CN, _T("Jiangxi")},
	{COUNTRY_CN, _T("Jilin")},
	{COUNTRY_CN, _T("Liaoning")},
	{COUNTRY_CN, _T("Qinghai")},
	{COUNTRY_CN, _T("Shaanxi")},
	{COUNTRY_CN, _T("Shandong")},
	{COUNTRY_CN, _T("Shanxi")},
	{COUNTRY_CN, _T("Sichuan")},
	{COUNTRY_CN, _T("Yunnan")},
	{COUNTRY_CN, _T("Zhejiang")},

	{COUNTRY_CN, _T("Guangxi")},
	{COUNTRY_CN, _T("Inner Mongolia")},
	{COUNTRY_CN, _T("Ningxia")},
	{COUNTRY_CN, _T("Xinjiang")},
	{COUNTRY_CN, _T("Tibet")},

	{COUNTRY_CN, _T("Beijing Municipality")},
	{COUNTRY_CN, _T("Chongqing Municipality")},
	{COUNTRY_CN, _T("Shanghai Municipality")},
	{COUNTRY_CN, _T("Tianjin Municipality")},

	{COUNTRY_CN, _T("Hong Kong")},
	{COUNTRY_CN, _T("Macau")},
	{COUNTRY_CN, _T("Taiwan")},
};

#define  NUMOFLIST			sizeof(g_map)/sizeof(g_map[0])


enum ErrCode{
	//file system error [1, 20]
	EC_AUDIT_FAILED = 1,
	//other error [51,+)
};

//	Plugin Implementation
//******************************************************************************
DFAPI_IMPLEMENT_PLUGIN_DLLMAIN()
DFAPI_IMPLEMENT_PLUGIN_ENTRYPOINTS(CMyPlugin)

//	CMyPlugin
//******************************************************************************
CMyPlugin::CMyPlugin()
{
	//Add the following codes to resolve the problem of many debug assertion dialog poped up by MFC library when debugging
#ifdef _DEBUG
	extern HINSTANCE __g_instance__;
	AFX_MODULE_STATE*  pModule = AfxGetModuleState();
	pModule->m_hCurrentInstanceHandle = __g_instance__;
	pModule->m_hCurrentResourceHandle = __g_instance__;
#endif
} 

CMyPlugin::~CMyPlugin()
{
}

//	plugin information
//------------------------------------------------------------------------------
DFAPI_VERSION_TYPE CMyPlugin::g_Version;
DFAPI_BUILDVERSION_TYPE CMyPlugin::g_BuildVersion;

void CMyPlugin::GetPluginInfoEx(LPCTSTR lpszLanguage, CString& strName, CString& strDescription, CString& strCopyright)
{
	strName = _T("Example Index");
	strDescription = _T("This an Example IDX Plugin for Kodak Capture Pro.");
	strCopyright = _T("Copyright (C) 1998-2008 Eastman Kodak Company.  All rights reserved.");
#ifdef	_DEBUG
	strName += _T("-(Debug)");
#endif	//_DEBUG
}

//	DFCFGACI
//------------------------------------------------------------------------------
CDFCFGACIBasicUI* CMyPlugin::CreateUI()
{
	return (new CMyCFGACIUI);
}

BOOL CMyPlugin::CFGACI_ValidateConfig(DFAPI_DATA_TYPE& config)
{
	// check the setting to see if it is a valid configuration or not.   
	CMyCFGACIData data;
	data.FromDataBlock(config);

	// Verify the config data is correct
	return data.IsValidData();
}

BOOL CMyPlugin::CFGACI_GetSupportedUIMode(BOOL& bSupportUI, BOOL& bSupportModalUI)
{
	// support embedded UI
	bSupportUI = TRUE;

	// Modal UI - not supported
	bSupportModalUI = FALSE;

	return TRUE;
}

BOOL CMyPlugin::CFGACI_GetUISize(int& nWidth, int& nHeight)
{
	// set the dialog size
	nWidth = 430;
	nHeight = 150;
	return TRUE;
}

BOOL CMyPlugin::CFGACI_ValidateUIConfig()
{
	// Validate the GUI configuration when press "OK" button 
	CMyCFGACIUI* pUI = (CMyCFGACIUI*) m_pUI;
	return pUI->ValidateUI();
}

//	KCIDXAPI
//------------------------------------------------------------------------------
DFAPI_PLUGIN_IMPLEMENT_STD_INTERFACE(CMyPlugin, KCIDXAPI_GetEditMode)
{
	// GetEditMode is where the Plugin Type is set.
	// In this example the type is based on the configuration
	// selected in setup.
	// In practice you would normally code this plugin to support 
	// only one mode. This example demonstrates all modes
	
	KCIDXAPI_GETEDITMODE_PARAM* p = (KCIDXAPI_GETEDITMODE_PARAM*) param;
	if (p == NULL) return FALSE;

	//Get config data
	CMyCFGACIData data;
	data.FromDataBlock(m_CurrentConfig);

	// Set the appropriate mode based on the setup data
	switch( data.m_lControlType )		// 0 = LIST, 1 = USER, 2 = ALLOWTYPING
	{
	case MODE_LIST:			// Index field is a dropdown list
		m_bValidate = FALSE;
		p->wMode = KCIDXAPI_EDITMODE_LIST;
		if( data.m_lAllowUserTyping )	// Can the user type an entry as well as choose from list
			p->wMode = KCIDXAPI_EDITMODE_LIST | KCIDXAPI_EDITMODE_ALLOWTYPING;
		break;
	case MODE_USER:			// Index field button will call a user function
		m_bValidate = FALSE;
		p->wMode = KCIDXAPI_EDITMODE_USER;
		if( data.m_lAllowUserTyping )
			p->wMode = KCIDXAPI_EDITMODE_USER | KCIDXAPI_EDITMODE_ALLOWTYPING;
		break;
	case MODE_ALLOWTYPING:	// Index field will look as normal
		m_bValidate = TRUE;
		p->wMode = KCIDXAPI_EDITMODE_ALLOWTYPING;
		break;
	}

	return TRUE;
}

DFAPI_PLUGIN_IMPLEMENT_STD_INTERFACE(CMyPlugin, KCIDXAPI_GetValueList)
{
	// KCIDXAPI_EDITMODE_LIST
	// p->wMode is set to KCIDXAPI_EDITMODE_LIST in KCIDXAPI_GetEditMode()

	// When KCIDXAPI_EDITMODE_List, Capture Pro will show the Index Field as a  
	// dropdown list from which an entry can be selected
	// if KCIDXAPI_GETEDITMODE_PARAM->dwMode is or'd with MODE_ALLOWTYPING, 
	// the user will also be able to type into this field

	// This function is called to populate the list prior to its display
	
	KCIDXAPI_GETVALUELIST_PARAM* p = (KCIDXAPI_GETVALUELIST_PARAM*) param;
	if( p == NULL ) 
		return FALSE;

	//Get config data
	CMyCFGACIData data;
	data.FromDataBlock(m_CurrentConfig);

	CString  strParameters;
	CString  strSeperator = _T("\r\n");

	// Using list mode - Fill the list with data
	State_Country* pItem = NULL;
	for (pItem = &g_map[0]; pItem < &g_map[NUMOFLIST]; pItem++) 
	{
		if (pItem->nCountry == COUNTRY_US) // set to COUNTRY_US in this example
		{
			strParameters.Append(pItem->pchState);
			strParameters.Append(strSeperator);
		}
	}

	// Now we have loaded the data into our list
	// get the length
	int nLen = strParameters.GetLength()*2+2;
	// Alloc data in our data block
	p->ValueList.Alloc(nLen);
	p->ValueList.Lock();
	// and copy the KCIDXAPI_GETVALUELIST_PARAM list
	wcscpy((LPTSTR)(p->ValueList.m_pData), (LPCTSTR)strParameters);
	p->ValueList.Unlock();

	return TRUE;
}

DFAPI_PLUGIN_IMPLEMENT_STD_INTERFACE(CMyPlugin, KCIDXAPI_GetValue)
{
	// KCIDXAPI_EDITMODE_USER
	// KCIDXAPI_GETEDITMODE_PARAM->wMode is set to KCIDXAPI_EDITMODE_USER 
	// in KCIDXAPI_GetEditMode() 
	// if KCIDXAPI_GETEDITMODE_PARAM->dwMode is or'd with MODE_ALLOWTYPING, 
	// the user will also be able to type into this field

	// When KCIDXAPI_EDITMODE_USER, Capture Pro will show a button to the 
	// right of the index field pressing the button will call this function.

	// In this function we will show a dialog with the list of 
	// states as the ListMode does.

	KCIDXAPI_GETVALUE_PARAM* p = (KCIDXAPI_GETVALUE_PARAM*) param;
	if( p == NULL ) 
		return FALSE;
 
	//Get config data
	CMyCFGACIData data;
	data.FromDataBlock(m_CurrentConfig);

//	CString sTemp;
//	sTemp.Format( _T("Data Block Control Type = %d"), data.m_lControlType );
//	MessageBox( GetFocus(), sTemp, _T("Debug"), MB_OK );

	CSelectStateDlg dlg;
	
	CStringList lst;
	State_Country* pItem = NULL;
	for (pItem = &g_map[0]; pItem < &g_map[NUMOFLIST]; pItem++) 
	{	// loop through the list and ad the states to the listbox
		if (pItem->nCountry == COUNTRY_US)	// set to COUNTRY_US in this example
		{	// add eah entry to the list
			lst.AddTail(pItem->pchState);
		}
	}

	// Display our dialog which will show the list
	dlg.InitList(lst);

	CString strSelectState;

	if (IDOK == dlg.DoModal())
		dlg.GetSelectItem( strSelectState );
	else 
		strSelectState.Empty();

	// Set the value of the selected entry
	p->szValue.Set( strSelectState );

	return TRUE;
}

DFAPI_PLUGIN_IMPLEMENT_STD_INTERFACE(CMyPlugin, KCIDXAPI_ValidateValue)
{
	// This function is called after an index is entered or edited
	// it will also be called when a batch is opened or output.
	
	// You cannot change the value of the data passed in this funtion call
/*
	KCIDXAPI_VALIDATEVALUE_PARAM* p = (KCIDXAPI_VALIDATEVALUE_PARAM*) param;
	if (p == NULL) return FALSE;

	// p->bIsValid	Set to TRUE if data value is valid
	// p->szValue	Copy of entered index value

	CString strValue(p->szValue.Get());

	// In this example we will only validate data if the mode is set to
	// KCIDXAPI_EDITMODE_USER (User has type an index value in)
	if( !m_bValidate )
	{	// In V1.1.2 up we have to put the value in the szValidatedVale
		// otherwise the index field value will be deleted
		p->bIsValid = TRUE;
		p->szValidatedValue = strValue;
		return TRUE;
	}

	// Get Config data
	CMyCFGACIData data;
	data.FromDataBlock(m_CurrentConfig);

	BOOL bValid = FALSE;
	
	// call our private Audit function
	if( !AuditUserEntry( strValue ) )
	{	// If it failes
		SetLastError(EC_AUDIT_FAILED);	// Set the error message
		p->bIsValid = FALSE;			// Indicate the data has an error
		return FALSE;
	}
	else
		p->bIsValid = TRUE;				// Data is valid

	// Ste the entered value in the 
	p->szValidatedValue = strValue;

	return TRUE;
*/
	KCIDXAPI_VALIDATEVALUE_PARAM* p = (KCIDXAPI_VALIDATEVALUE_PARAM*) param;
	if (p == NULL)
		return FALSE;

	// p->bIsValid Set to TRUE if data value is valid
	// p->szValue Copy of entered index value

	// In this example we will only validate data if the mode is set to
	// KCIDXAPI_EDITMODE_USER (User has manually typed an index value)

	// Get the index data
	CString strValue(p->szValue.Get());

	if( !m_bValidate )
	{
		// new from v1.1.2
		// Must put the original value into szValidateValue member other
		// wise index will be deleted
		p->szValidatedValue = strValue;
		p->bIsValid = TRUE;
		return TRUE;
	}

	BOOL bValid = FALSE;
	
	// call our private Audit function
	if( !AuditUserEntry( strValue ) )
	{	// If it fails
		SetLastError(EC_AUDIT_FAILED); // Set the error message
		p->bIsValid = FALSE; // Indicate the data has an error
		return FALSE;
	} 
	else
	{
		p->bIsValid = TRUE; // Data is valid
		// new from v1.1.2
		// Must put the original value into szValidateValue member other
		// wise index will be deleted
		p->szValidatedValue = strValue;
	}

	return TRUE;



}


DFAPI_PLUGIN_IMPLEMENT_STD_INTERFACE(CMyPlugin, KCIDXAPI_FormatOutput)
{
	// FormatOutput is called after the ValidateValue function during
	// the 'Process' phase of processing. It is not called during 
	// normal index funtionality.
	// In this function you have the option for format the data, if required
	// This function is the only place that you can change the index data
	// if you are using an edit mode of KCIDXAPI_EDITMODE_USER
	
	KCIDXAPI_FORMATOUTPUT_PARAM* p = (KCIDXAPI_FORMATOUTPUT_PARAM*) param;
	if (p == NULL) return FALSE;

	// Get Config data
	CMyCFGACIData data;
	data.FromDataBlock(m_CurrentConfig);

	// p->szInputValue		Copy of the data as entered in the index field
	// p->szOutputValue		Formated data to be output

	// This example will simply add a '-0' to the end of the index value 
	CString strInput(p->szInputValue.Get());

	// append '-0' to the value
	strInput += _T("-0");

	// and set it back to the index value to be output
	p->szOutputValue.Set( strInput );

	return TRUE;
}


BOOL CMyPlugin::AuditUserEntry( CString &sEntry )
{
	// Private Audit funtion
	if( sEntry.Left(2) != "V8" ) // data must start with "V8"
		return FALSE;

	return TRUE;
}

BOOL CMyPlugin::GetErrorMsg(LPCTSTR lpszLanguage, DWORD dwErrCode, CString& strMsg)
{		
	// If you encountered an error we called SetLastError with our Error No and then returned FALSE.
	// After the PlugIn has retruning FALSE Capture Pro will call this function to get the error description.
	// you should populate the CString& strMsg with your error message
	// IMPORTANT
	// You should NOT display a MessageBox in this fuction

	switch (dwErrCode)
    {
    case EC_AUDIT_FAILED:
        strMsg = _T("Audit failed! Data must start with 'V8'");
        break;
	default:
		    strMsg = _T("Undefined index failure");
        break;
    }
    return TRUE;
}