/*!
	@file	DIIMFAPI.h
	@author	Max Wu (max.wu@kodak.com)
	@author	Copyright (C) 1998-2007 Eastman Kodak Company.
	@author	All rights reserved.
	@date	2006-10-18
*/

#pragma once

#include "DFShellAPI.h"

//	DIIMFAPI
//******************************************************************************
#define	DIIMFAPI											0x00002440

//	DIIMFAPI_VERSION
//******************************************************************************
#define	DIIMFAPI_VERSION									0x00010000

//	DIIMFAPI_MODEMASK	
//******************************************************************************
#define	DIIMFAPI_MODEMASK_IMAGE_BITONAL						0x00000001		//!< support bitonal images
#define	DIIMFAPI_MODEMASK_IMAGE_GRAYSCALE					0x00000002		//!< support grayscale images
#define	DIIMFAPI_MODEMASK_IMAGE_COLOR						0x00000004		//!< support color images
#define	DIIMFAPI_MODEMASK_SOURCE_IMAGEFILE					0x00000100		//!< support AddImageFile()
#define	DIIMFAPI_MODEMASK_SOURCE_IMAGE						0x00000200		//!< support AddImage()
#define	DIIMFAPI_MODEMASK_GROUP_LEVEL1						0x00010000		//!< support group at level 1 (typically image level)
#define	DIIMFAPI_MODEMASK_GROUP_LEVEL2						0x00020000		//!< support group at level 2 (typically page level)
#define	DIIMFAPI_MODEMASK_GROUP_LEVEL3						0x00040000		//!< support group at level 3 (typically document level)
#define	DIIMFAPI_MODEMASK_GROUP_LEVEL4						0x00080000		//!< support group at level 4 (typically batch level)
#define	DIIMFAPI_MODEMASK_GROUP_LEVEL5						0x00100000		//!< support group at level 5 (typically job level)
#define	DIIMFAPI_MODEMASK_GROUP_LEVEL6						0x00200000		//!< support group at level 6 (typically sys level)
#define	DIIMFAPI_MODEMASK_SINGLE_IMAGE						0x00010000		//!< support single image file
#define	DIIMFAPI_MODEMASK_MULTIPLE_IMAGES					0xFFFE0000		//!< support multi-image file
#define DIIMFAPI_MODEMASK_FORPROCESSMODEIMGS				0x00000010		//!< DIIMFAPI_DV_CURRENTMODE used for process, all images in one file
#define	DIIMFAPI_MODEMASK_FORPROCESSMODEDOC					0x00000020		//!< DIIMFAPI_DV_CURRENTMODE used for process by doc
#define DIIMFAPI_MODEMASK_FORPROCESSMODEBAT					0x00000040      //!< DIIMFAPI_DV_CURRENTMODE used for process by batch

//	DIIMFAPI_RENAMEROUTINE
//******************************************************************************
/*!	
	BOOL RenameRoutine(DFAPI_TEXTPATH_TYPE& szOldFilePath,
					   DFAPI_TEXTPATH_TYPE& szNewFilePath,
					   LPVOID lpUserData);
*/
typedef BOOL (*DIIMFAPI_RENAMEROUTINE)(DFAPI_TEXTPATH_TYPE&, DFAPI_TEXTPATH_TYPE&, LPVOID);

//	DIIMFAPI_IMAGESIDE
//******************************************************************************
#define	DIIMFAPI_IMAGESIDE_UNKNOWN							0x0000			//!< unknown
#define	DIIMFAPI_IMAGESIDE_FRONT							0x0001			//!< front side
#define	DIIMFAPI_IMAGESIDE_REAR								0x0002			//!< rear side
#define	DIIMFAPI_IMAGESIDE_BOTH								0x0003			//!< both sides, eg. merged image with both sides

//	DIIMFAPI_IMAGEINFO
//******************************************************************************
struct DIIMFAPI_IMAGEINFO{
	DWORD							dwID;					//!< image ID, start from 1
	WORD							wScannedSide;			//!< original side
	WORD							wCurrentSide;			//!< current side
	DWORD							dwHWPartNo;				//!< hardware part number, splitted by hardware
	DWORD							dwSWPartNo;				//!< software part number, splitted by software
	DWORD							dwFlags;				//!< flags, reserved
};

//	DIIMFAPI_GETDEFAULTFILEEXT
//******************************************************************************
#define	DIIMFAPI_GETDEFAULTFILEEXT_PROCNAME					_T("DIIMFAPI.GetDefaultFileExt")
struct DIIMFAPI_GETDEFAULTFILEEXT_PARAM{
	DFAPI_TEXT32_TYPE				szFileExt;				//!< [out]: default file extension, eg. ".dat"
};

//	DIIMFAPI_GETDEFAULTFILEICON
//******************************************************************************
#define	DIIMFAPI_GETDEFAULTFILEICON_PROCNAME				_T("DIIMFAPI.GetDefaultFileIcon")
struct DIIMFAPI_GETDEFAULTFILEICON_PARAM{
	__int32							nDesiredWidth;			//!< [in]: desired width of the icon
	__int32							nDesiredHeight;			//!< [in]: desired height of the icon
	HICON							hIcon;					//!< [out]: default file icon, invoker will be responsible for releasing
};

//	DIIMFAPI_RESET
//******************************************************************************
#define	DIIMFAPI_RESET_PROCNAME								_T("DIIMFAPI.Reset")
struct DIIMFAPI_RESET_PARAM{
};

//	DIIMFAPI_BEGINIMAGESET
//******************************************************************************
#define	DIIMFAPI_BEGINIMAGESET_PROCNAME						_T("DIIMFAPI.BeginImageSet")
struct DIIMFAPI_BEGINIMAGESET_PARAM{
	__int32							nLevel;					//!< [in]: level of image set, start from 1
	DFAPI_TEXT128_TYPE				szLevelName;			//!< [in]: level name, eg. "image", "page", "document", etc.
	DFAPI_TEXT128_TYPE				szImageSetName;			//!< [in]: image set name
	DFAPI_TEXTPATH_TYPE				szFilePath;				//!< [in]: image set file path
};

//	DIIMFAPI_ADDIMAGEFILE
//******************************************************************************
#define	DIIMFAPI_ADDIMAGEFILE_PROCNAME						_T("DIIMFAPI.AddImageFile")
struct DIIMFAPI_ADDIMAGEFILE_PARAM{
	DIIMFAPI_IMAGEINFO				info;					//!< [in]: image information
	DFAPI_TEXTPATH_TYPE				szImageFilePath;		//!< [in]: image file path
};

//	DIIMFAPI_ADDIMAGE
//******************************************************************************
#define	DIIMFAPI_ADDIMAGE_PROCNAME							_T("DIIMFAPI.AddImage")
struct DIIMFAPI_ADDIMAGE_PARAM{
	DIIMFAPI_IMAGEINFO				info;					//!< [in]: image information
};

//	DIIMFAPI_ENDIMAGESET
//******************************************************************************
#define	DIIMFAPI_ENDIMAGESET_PROCNAME						_T("DIIMFAPI.EndImageSet")
struct DIIMFAPI_ENDIMAGESET_PARAM{
	__int32							nLevel;					//!< [in]: level of image set, start from 1
};

//	DIIMFAPI_LEGACYDATAIO
//******************************************************************************
#define	DIIMFAPI_LEGACYDATAIO_PROCNAME						_T("DIIMFAPI.LegacyDataIO")
struct DIIMFAPI_LEGACYDATAIO_PARAM{
	BOOL							bGetCurrentModeValid;	//!< [in]: flag to show whether get current mode is valid
	DWORD							dwGetCurrentMode;		//!< [out]: get current mode
};

//	DFENVACI - Variables
//******************************************************************************
#define	DIIMFAPI_TV_SIGNATURE_USERNAME						0		//!< signature - user name	
#define	DIIMFAPI_TV_OUTPUTLISTFILEPATH						1		//!< output list file path

#define	DIIMFAPI_DV_SUPPORTEDMODE							0		//!< supported mode, typically readonly
#define	DIIMFAPI_DV_CURRENTMODE								1		//!< current mode
#define	DIIMFAPI_DV_MAXFILESIZE								2		//!< max file size in bytes, 0 means no limit
#define	DIIMFAPI_DV_MAXFILESIZERENAMEROUTINE				3		//!< max file size rename routine
#define	DIIMFAPI_DV_MAXFILESIZERENAMEROUTINEDATA			4		//!< max file size rename routine data
#define DIIMFAPI_DV_SIGNATURE_SERIALNUMBER					5		//!< signature - serial number
