/*!
	@file	DIIMFAPIPlugin.h
	@author	Max Wu (max.wu@kodak.com)
	@author	Copyright (C) 1998-2007 Eastman Kodak Company.
	@author	All rights reserved.
	@date	2006-10-18
*/

#pragma once

#include "DIIMFAPI.h"
#include "DFShellAPIPlugin.h"
#include "DFENVACIBasicImpl.h"
#include "DFERRACIBasicImpl.h"
#include "DFCFGACIBasicImpl.h"

//	TDIIMFAPIPlugin
//******************************************************************************
template <class SESSIONCLASS, class PLUGINCLASS>
class TDIIMFAPIPlugin:public TDFShellAPIPlugin<SESSIONCLASS, PLUGINCLASS>,
					  public CDFENVACIBasicImpl,
					  public CDFERRACIBasicImpl,
					  public CDFCFGACIBasicImpl{
	//!	@name	plugin information
	//@{
public:
	static DFAPI_TYPEID_TYPE		g_Type;					//!< API type ID
	static DFAPI_VERSION_TYPE		g_APIVersion;			//!< API version
	//@}

	//!	@name	session
	//@{
public:
	virtual void OnOpenSession(DFAPI_SESSION session);
	virtual void OnCloseSession(DFAPI_SESSION session){}
	//@}

	//!	@name	shell API
	//@{
public:
	virtual BOOL GetProcAddress(DFAPI_SESSION session, LPCTSTR lpszProcName, LPVOID* lpProcAddress);
	virtual void CloseSession(DFAPI_SESSION session){}
	//@}

	//!	@name	DIIMFAPI
	//@{
	DFAPI_PLUGIN_STD_INTERFACE(PLUGINCLASS, DIIMFAPI_GetDefaultFileExt)
	DFAPI_PLUGIN_STD_INTERFACE(PLUGINCLASS, DIIMFAPI_GetDefaultFileIcon)
	DFAPI_PLUGIN_STD_INTERFACE(PLUGINCLASS, DIIMFAPI_Reset)
	DFAPI_PLUGIN_STD_INTERFACE(PLUGINCLASS, DIIMFAPI_BeginImageSet)
	DFAPI_PLUGIN_STD_INTERFACE(PLUGINCLASS, DIIMFAPI_AddImageFile)
	DFAPI_PLUGIN_STD_INTERFACE(PLUGINCLASS, DIIMFAPI_AddImage)
	DFAPI_PLUGIN_STD_INTERFACE(PLUGINCLASS, DIIMFAPI_EndImageSet)
	//@}

	//!	@name	DIIMFAPI - LegacyDataIO
	//@{
public:
	static BOOL __DIIMFAPI_LegacyDataIO(DFAPI_SESSION session, LPVOID param){
		PLUGINCLASS* pThis = (PLUGINCLASS*) CheckSession(session);
		if (pThis == NULL) return FALSE;
		return pThis->DIIMFAPI_LegacyDataIO(session, param);
	}
protected:
	virtual BOOL DIIMFAPI_LegacyDataIO(DFAPI_SESSION session, LPVOID param){
		return TRUE;
	}
	//@}

	//!	@name	DFENVACI - variables
	//@{
protected:
	DFENVACI_PLUGIN_TV_GET(DIIMFAPI_TV_SIGNATURE_USERNAME, TV_GetSignatureUserName)
	DFENVACI_PLUGIN_TV_GET(DIIMFAPI_TV_OUTPUTLISTFILEPATH, TV_GetOutputListFilePath)
	DFENVACI_PLUGIN_DV_SET(DIIMFAPI_DV_SUPPORTEDMODE, DV_SetSupportedMode, DWORD)
	DFENVACI_PLUGIN_DV_GET(DIIMFAPI_DV_CURRENTMODE, DV_GetCurrentMode, DWORD&)
	DFENVACI_PLUGIN_DV_SET(DIIMFAPI_DV_MAXFILESIZE, DV_SetMaxFileSize, DWORD)
	DFENVACI_PLUGIN_DV_GET(DIIMFAPI_DV_MAXFILESIZE, DV_GetMaxFileSize, DWORD&)
	DFENVACI_PLUGIN_DV_GET(DIIMFAPI_DV_SIGNATURE_SERIALNUMBER, DV_GetSignatureSerialNumber, DWORD&)
	DFENVACI_PLUGIN_DV_GET(DIIMFAPI_DV_MAXFILESIZERENAMEROUTINE, DV_GetMaxFileSizeRenameRoutine, DIIMFAPI_RENAMEROUTINE&)
	DFENVACI_PLUGIN_DV_GET(DIIMFAPI_DV_MAXFILESIZERENAMEROUTINEDATA, DV_GetMaxFileSizeRenameRoutineData, LPVOID&)
	//@}
};

//******************************************************************************
#include "DIIMFAPIPlugin.inl"
